﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using Serilog;
using System.Diagnostics;
using System.Security.Policy;
using System.Text.Json;

namespace iFileProxy.Services
{
    /// <summary>
    /// 下载任务管理器
    /// </summary>
    public class TaskManager
    {
        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<TaskManager>();
        private readonly AppConfig? _appConfig = AppConfig.GetCurrConfig("iFileProxy.json");
        private readonly DatabaseHelper _dbHelper;
        private Dictionary<string, TaskInfo> runningTasks = [];
        public TaskManager()
        {
            _logger.Information("Initializing TaskManager...");
            if (_appConfig != null)
                _dbHelper = new DatabaseHelper(_appConfig);
            else
            {
                _logger.Fatal($"Failed to load application configuration");
                Environment.Exit(1);
            }
            _logger.Information("TaskManager init succ.");
        }
        /// <summary>
        /// 添加一个新的下载任务
        /// </summary>
        /// <param name="c">HttpContext</param>
        /// <returns></returns>
        public TaskAddState AddTask(HttpContext c)
        {
            string? clientIp = MasterHelper.GetClientIPAddr(c);
            string? t_url = c.Request.Query["url"].FirstOrDefault() ?? c.Request.Form["url"].FirstOrDefault();

            if (_appConfig.DownloadOptions.MaxParallelTasks != 0 && runningTasks.Count >= _appConfig.DownloadOptions.MaxParallelTasks)
                return TaskAddState.ErrMaxParallelTasksLimit;

            if (runningTasks.Where(x => x.Value.Url == t_url).Any())
                return TaskAddState.ErrUrlRepeat;

            if (!MasterHelper.CheckUrlIsValid(t_url))
                return TaskAddState.ErrUrlInvalid;

            if (_appConfig.SecurityOptions.BlockedHost.IndexOf(new Uri(t_url).Host) != -1)
                return TaskAddState.ErrTargetHostForbidden;

            DownloadFileInfo? fileInfo = FileDownloadHelper.GetDownloadFileInfo(t_url);
            if (fileInfo == null)
                return TaskAddState.ErrGetFileInfo;

            if (fileInfo.Size == 0 || fileInfo.Size > _appConfig.DownloadOptions.MaxAllowedFileSize)
                return TaskAddState.ErrFileSizeLimit;

            if (_appConfig.SecurityOptions.BlockedFileName.IndexOf(fileInfo.FileName) != -1)
                return TaskAddState.ErrFileNameForbidden;

                TaskInfo taskInfo = new()
            {
                Url = t_url,
                TaskId = Guid.NewGuid().ToString(),
                AddTime = DateTime.Now,
                FileName = fileInfo.FileName,
                ClientIp = clientIp,
                Size = fileInfo.Size,
                Status = TaskState.NoInit,
                UpdateTime = DateTime.Now
            };
            if (_dbHelper.InsertTaskData(taskInfo))
            {
                StartTask(taskInfo);
                taskInfo.Status = TaskState.Running;
                _dbHelper.UpdateTaskStatus(taskInfo);
                _logger.Debug("数据插入成功");
                return TaskAddState.Success;
            }
            else
                return TaskAddState.ErrDbFail;
        }

        public async void StartTask(TaskInfo taskInfo)
        {
            if (runningTasks.ContainsKey(taskInfo.TaskId))
            {
                _logger.Error($"指定的task已经存在!!!");
                return;
            }
            Process aria2c = new();
            aria2c.StartInfo = new ProcessStartInfo
            {
                FileName = _appConfig.DownloadOptions.Aria2cPath,
                WorkingDirectory = _appConfig.DownloadOptions.SavePath,
                Arguments = $"-x {_appConfig.DownloadOptions.ThreadNum} -s {_appConfig.DownloadOptions.ThreadNum} {taskInfo.Url}",
                RedirectStandardOutput = true,
                RedirectStandardError = true,
                RedirectStandardInput = true,
                UseShellExecute = false,
                Environment = { { "TaskId", taskInfo.TaskId } }
            };
            try
            {
                aria2c.Start();
                aria2c.BeginOutputReadLine();
                aria2c.BeginErrorReadLine();
                aria2c.OutputDataReceived += Aria2c_OutputDataReceived;
                aria2c.ErrorDataReceived += Aria2c_ErrorDataReceived;
                runningTasks.Add(taskInfo.TaskId, taskInfo);
                await aria2c.WaitForExitAsync();
                runningTasks.Remove(taskInfo.TaskId);

                if (aria2c.ExitCode != 0)
                {
                    _logger.Error($"task: {taskInfo.TaskId} 进程退出状态异常 ExitCode: {aria2c.ExitCode}");
                    taskInfo.Status = TaskState.Error;
                    _dbHelper.UpdateTaskStatus(taskInfo);
                }
                else
                {
                    taskInfo.Status = TaskState.End;
                    _dbHelper.UpdateTaskStatus(taskInfo);
                    taskInfo.Hash = MasterHelper.GetFileHash(Path.Combine(_appConfig.DownloadOptions.SavePath, taskInfo.FileName), FileHashAlgorithm.MD5);
                    _dbHelper.UpdateTaskHash(taskInfo);
                }
            }
            catch (Exception)
            {
                _logger.Fatal("执行下载任务时候出现致命问题");
                throw;
            }
        }

        private void Aria2c_ErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Error($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        private void Aria2c_OutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Debug($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        public List<TaskInfo> GetTaskListByIpAddr(HttpContext c)
        {
            string? clientIp = MasterHelper.GetClientIPAddr(c);
            return JsonSerializer.Deserialize<List<TaskInfo>>(_dbHelper.GetTaskListByIP(clientIp)) ?? [];
        }

        //public bool DeleteTask(HttpContext c)
        //{

        //}

        //public bool UpdateTaskStatus(HttpContext c)
        //{
        //}
        //public List<TaskInfo> GetAllTaskInfo(HttpContext c)
        //{

        //}

        //public TaskInfo GetTaskInfo(HttpContext c)
        //{

        //}
    }
}
